//Pins for the LED and LDR
int led = 3;
int ldr = A0;

//Pins for RGB led
int redPin = 5;
int bluePin = 9;
int greenPin = 6;

//Pin for buzzer
int buzzer = 12;

bool buzzerOn = false; //Tracks buzzer status

int on_limit =  15; //buzzer turns on if brightness > 15
int off_limit = 5; //buzzer turns off if brightness > 15

#include <LCD_I2C.h>
LCD_I2C lcd(0x27);

float w = 0.18778660479841863; //slope from the learning model(Linear regression model)
float b = -33.98484004573112 ;//intercept from the learning model(Linear regression model)

void setup() 
{
pinMode(led, OUTPUT); //LED as output
pinMode(ldr, INPUT);  //LDR as input

lcd.begin();
lcd.backlight();

//RGB pins as output
pinMode(redPin, OUTPUT);
pinMode(greenPin, OUTPUT);
pinMode(bluePin, OUTPUT);

//Buzzer as output 
pinMode(buzzer, OUTPUT);

Serial.begin(9600);
}

void setColor(int red, int green, int blue)
{
red = 255 - red;
green = 255 - green;
blue = 255 - blue;

analogWrite(redPin, red);
analogWrite(greenPin, green);
analogWrite(bluePin, blue);
}

//The buzzer beeps once
void beep() 
{
digitalWrite(buzzer, HIGH);
delay(150);       
digitalWrite(buzzer, LOW);
delay(150);       
}

void loop() 
{
float ldrValue = analogRead(ldr); //Reads the value from the ldr sensor and stores it
float prediction = w*ldrValue + b; //Calculates the predicted brightness(Linear regression model) and stores it

int brightness = prediction;

bool on = buzzerOn;

if(!on && brightness > on_limit)
{
on = true; //Led turned on
}
else if(on && brightness < off_limit)
{
on = false; //led turned off
}

if(on != buzzerOn) 
{
beep(); //beeps if led changed status (on/off)

buzzerOn = on; //updates ot
}

//Limits the led brightness to 255 - 0
if (prediction > 255)
{
prediction = 255;
}

if (prediction < 0)
{
prediction = 0;
}

analogWrite(led,prediction); //Outputs the brightness of the LED with the predicted brightness

int brightnessPercent = (prediction / 255.0) * 100; //Brightness into percentage

lcd.setCursor(0, 0);
lcd.print("Brightness:");
lcd.setCursor(0, 1);
lcd.print("      ");  //clear previous number (2nd row)
lcd.setCursor(0, 1);
lcd.print(brightnessPercent);
lcd.print("%");

if (prediction > 10) 
{
setColor(0, 255, 0); //Green when LED is on
} 
else 
{
setColor(255, 0, 0); //Red when LED is off
}


if(Serial.available()) //checking if data is coming from python
{
char command = Serial.read(); //Reads it
if(command == 's') //Checks if lamp is on
{
  if (prediction > 10)
  {
  Serial.println("on"); //Sends 'on' to python
  }

  else
  {
  Serial.println("off"); //Sends 'off' to python
  }
}

delay(10); //delay before the loop starts again
}
}


